@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Author:   $Author: merkosh $
-# Revision: $Rev: 137 $
+# Revision: $Rev: 145 $
 ############################################################################
 #    Copyright (C) 2005 by Uwe Mayer                                       #
 #    merkosh@hadiko.de                                                     #
@@ -161,7 +161,6 @@ class ListViewRecord(QListView):
             # all other fields: if no parent exists yet
             if (not self.groupMap.has_key(value)):
                 self.log.debug("Creating parent group %s", value)
-#                parent = QListViewItem(self, value, "(1)")
                 parent = ListViewSort(self, value, "(1)")
                 parent.setSelectable(False)
                 self.groupMap[value] = parent
@@ -189,7 +188,6 @@ class ListViewRecord(QListView):
             if (not isinstance(r, AbstractFileRecord)):
                 raise(TypeError("Record not instance of AbstractFileRecord"))
             parent = self.parentGroup(r)
-#            obj = QListViewItem(parent, *self.compileTitles(r) )
             obj = ListViewSort(parent, *self.compileTitles(r) )
             obj.setRenameEnabled(0, True)
             self.items[obj] = r
@@ -232,15 +230,45 @@ class ListViewRecord(QListView):
                         'translatedTitle', 'director', 'producer', 'country',
                         'category', 'actors', 'description', 'comments',
                         'languages', 'subtitles', 'year')
+
+        noRecord = dict()
+        for f in allFieldKeys: noRecord.setdefault(f, "")
+        
         keyFound = None
         found = False
         visited = dict()
         obj = self.currentItem()        # start at current item
+
+
+        def stepDirection(obj, directionForward):
+            """given a listviewitem object it takes a step and returns the next element"""
+            # searching forward
+            if (directionForward):
+                # step one down
+                obj = obj.itemBelow()
+                # stepped on parent group element: goto first child
+                if (obj != None) and (not self.items.has_key(obj)):
+                    obj = obj.firstChild()
+            # searching backwards
+            else:
+                # step one upwards
+                obj = obj.itemAbove()
+                # stepped on parent group element
+                if ((obj != None) and (not self.items.has_key(obj))):
+                    # get previous element (may be group or node)
+                    obj = obj.itemAbove()
+                    # if is group element
+                    if ((obj != None) and (not self.items.has_key(obj))):
+                        # goto last child
+                        obj = obj.firstChild()
+                        assert(obj != None) # every group element has children
+                        while (obj.nextSibling() != None): obj = obj.nextSibling()
+            return obj
         
+
         # on first time use current item as starting point
-        if (not self.firstSearch):
-            if (options['searchForward']): obj = obj.itemBelow()
-            else: obj = obj.itemAbove()
+        if ((obj != None) and (not self.firstSearch)):
+            obj = stepDirection(obj, options['searchForward'])
         self.firstSearch = False
 
         # considder case-sensitivity
@@ -256,7 +284,8 @@ class ListViewRecord(QListView):
                     obj = self.firstChild()
                 else:
                     obj = self.lastItem()
-                    
+                    assert(self.items.has_key(obj))
+
             # at end of page (top or bottom)
             if (obj == None):
                 QApplication.beep()
@@ -274,7 +303,7 @@ class ListViewRecord(QListView):
                 return
 
             # search in all fields
-            rec = self.items[obj]            
+            rec = (self.items.has_key(obj) and self.items[obj]) or noRecord
             if (field == None):
                 for k in allFieldKeys:
                     keyFound = k
@@ -286,7 +315,7 @@ class ListViewRecord(QListView):
             # search in given field
             else:
                 assert(rec.has_key(field))
-                value = rec[field]
+                value = unicode(rec[field])
                 if (not options['caseSensitive']):
                     value = value.lower()
                 found = (value.find(text) != -1)
@@ -296,8 +325,7 @@ class ListViewRecord(QListView):
 
             # move on in direction
             if (not found):
-                if (options['searchForward']): obj = obj.itemBelow()
-                else: obj = obj.itemAbove()
+                obj = stepDirection(obj, options['searchForward'])
 
         # focus on found object
         if (found):
@@ -408,8 +436,9 @@ class ListViewRecord(QListView):
 
     def keyPressEvent(self, e):
         """reacs on key-press-events for incremental search"""      
-        # ignore keypressed when renaming
-        if (self.isRenaming()):
+        # ignore keypresses when renaming
+        # ignore keypresses when empty
+        if (self.isRenaming() or (len(self) == 0)):
             e.ignore()
             return
 
