@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Author:   $Author: merkosh $
-# Revision: $Rev: 123 $
+# Revision: $Rev: 160 $
 ############################################################################
 #    Copyright (C) 2005 by Uwe Mayer                                       #
 #    merkosh@hadiko.de                                                     #
@@ -49,7 +49,7 @@ from Settings import getPreferences
 #-- constants ------------------------------------------------------------------
 # defines maximum number of characters which are recognized as a file suffix
 # rather than binary image data
-MAX_SUFFIX_LENGTH = 20
+MIN_PICTURE_SIZE = 20
 
 
 recycle_bin_data = \
@@ -314,18 +314,24 @@ class PicturePreview(PicturePreviewBase):
     def mousePressEvent(self, e):
         """called when a mouse button is pressed"""
         # only accept left mouse button
-        if ((e.button() == e.RightButton) and (self.img != None)):
+        if (e.button() == e.RightButton):
             menu = QPopupMenu(self)
             menu.insertItem(self.tr("Picture Menu"), None)
             menu.insertSeparator()
-            menu.insertItem(QIconSet(self.copy_pix), self.tr("Copy"), self.copyPicture)
+            if (self.img): menu.insertItem(QIconSet(self.copy_pix), self.tr("Copy"), self.copyPicture)
             menu.insertItem(QIconSet(self.paste_pix), self.tr("Paste"), self.pastePicture)
-            menu.insertItem(QIconSet(self.saveAs_pix), self.tr("Save As"), self.saveAsPicture)
-            menu.insertItem(QIconSet(self.recycle_pix), self.tr("Delete"), self.deletePicture)
-            if (self.copyImg == None):
-                menu.setItemEnabled(menu.idAt(3), False)
-            if (self.img == None):
-                menu.setItemEnabled(menu.idAt(4), False)
+            if (self.img): menu.insertItem(QIconSet(self.saveAs_pix), self.tr("Save As"), self.saveAsPicture)
+            if (self.img): menu.insertItem(QIconSet(self.recycle_pix), self.tr("Delete"), self.deletePicture)
+
+            if (self.img):
+                if (self.copyImg == None):
+                    menu.setItemEnabled(menu.idAt(3), False)
+                if (self.img == None):
+                    menu.setItemEnabled(menu.idAt(4), False)
+            else:
+                if (self.copyImg == None):
+                    menu.setItemEnabled(menu.idAt(2), False)
+                    
             menu.exec_loop(QCursor.pos())
             return
             
@@ -337,6 +343,11 @@ class PicturePreview(PicturePreviewBase):
 
     def saveAsPicture(self):
         """saves the picture as a file"""
+        # skip out when no picture available
+        if (not self.img):
+            self.log.warning("Not saving picture: no picture available")
+            return
+        
         # display dialog
         filename = QFileDialog.getSaveFileName(
             pref['defaultPath'], self.tr("Images (*.bmp, *.jpeg, *.pbm, *.png, *.ppm *.xbm, *.xpm)"),
@@ -372,13 +383,21 @@ class PicturePreview(PicturePreviewBase):
         self.log.info("Deleting picture from record")
         self.img = None                 # remove picture
 
-        # remove linked image
-        filename = fjoin(self.basedir, self.record['picture'])
-        if (self.record['picture'] and fexists(filename)):
-            try:
-                os.unlink(filename)
-            except OSError, e:
-                self.log.error("Could not remove linked file: %s", unicode(e))
+        # query wether picture should be removed from filesystem (linked image only)
+        if (len(self.record['embeddedImage']) < MIN_PICTURE_SIZE):
+            ans = QMessageBox.question(self, self.tr("delete from file system"), self.tr("Delete picture from filesystem?"),
+                                       QMessageBox.Yes, QMessageBox.No)
+            if (ans == QMessageBox.Yes):
+                self.log.info("Deleting image from filesystem")
+                # remove linked image
+                filename = fjoin(self.basedir, self.record['picture'])
+                if (self.record['picture'] and fexists(filename)):
+                    try:
+                        os.unlink(filename)
+                    except OSError, e:
+                        self.log.error("Could not remove linked file: %s", unicode(e))                        
+                else:
+                    self.log.warning("Not removing image from filesystem: file not found")
 
         # decode and display changes
         self.imageDecode()              # decode to empty picture
@@ -387,11 +406,15 @@ class PicturePreview(PicturePreviewBase):
 
     def copyPicture(self):
         """remembers an image from a record"""
+        if (self.img == None):
+            self.log.warning("Not copying picture: no picture available")
         self.copyImg = self.img
         
 
     def pastePicture(self):
         """pasts a copied image to the record"""
+        if (self.copyImg == None):
+            self.log.warning("Not pasting picture: no picture available")
         self.onImageDropped(self.copyImg)
         
 
@@ -434,7 +457,12 @@ class PicturePreview(PicturePreviewBase):
         if (pref['embeddedImage']):
             # extract image from QImage 
             data = IODeviceBuffer()
-            if ( self.img.save(data, pref['imageFormat']) ):
+            
+            if (self.img == None):
+                self.log.debug("Saving empty image")
+                self.record['embeddedImage'] = ""
+                self.record['picture'] = u""                
+            elif ( self.img.save(data, pref['imageFormat']) ):
                 self.log.debug("Saving embedded image: %d bytes", data.size())
                 self.record['embeddedImage'] = data.readAll()
                 self.record['picture'] = ".%s"%pref['imageFormat'].lower()
@@ -482,7 +510,10 @@ class PicturePreview(PicturePreviewBase):
             data = IODeviceBuffer()
             # copy to target buffer is successfull
             if (self.img == None):
-                self.log.info("No picture data to save")
+                self.log.info("Saving empty image")
+                self.record['embeddedImage'] = ""
+                self.record['picture'] = u""                              
+                
             elif (self.img.save(data, pref['imageFormat'])):
                 self.log.debug("Saving linked image %s: %d bytes", abs_filename, data.size())
                 try:
@@ -502,6 +533,9 @@ class PicturePreview(PicturePreviewBase):
                 self.log.error("Could not save image to intermediate buffer")
                 self.record['embeddedImage'] = ""
                 self.record['picture'] = u""
+
+        # notify of modification
+        self.emit(PYSIGNAL("modified"), ())
         
         
     def setRecord(self, rec):
@@ -515,7 +549,7 @@ class PicturePreview(PicturePreviewBase):
             self.img = None
             
         # record has embedded image   
-        elif (len(self.record['embeddedImage']) > MAX_SUFFIX_LENGTH):
+        elif (len(self.record['embeddedImage']) > MIN_PICTURE_SIZE):
             self.log.debug("Loading embedded image")
             self.img = QImage()
             ok = self.img.loadFromData(self.record['embeddedImage'])
