@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Author:   $Author: merkosh $
-# Revision: $Rev: 128 $
+# Revision: $Rev: 159 $
 ############################################################################
 #    Copyright (C) 2005 by Uwe Mayer                                       #
 #    merkosh@hadiko.de                                                     #
@@ -42,9 +42,6 @@ from PicturePreview import PicturePreview
 from Settings import getPreferences
 from ListViewRecord import ListViewRecord
 
-# TODO: use C++ compiled version
-#from Qt.QListViewItemNumeric.QListViewItemNumeric import QListViewItemNumeric
-
 
 #-- preferences ----------------------------------------------------------------
 pref = getPreferences(__name__)
@@ -93,7 +90,8 @@ class RecordList(RecordListBase):
         # connect signals
         self.connect( self.listViewRecord, SIGNAL("currentChanged(QListViewItem*)"), self.onCurrentChanged )
         self.connect( self.listViewRecord, PYSIGNAL("searchTextChanged"), PYSIGNAL("searchTextChanged"))
-        self.connect( self.form, PYSIGNAL("modified()"), self.listViewRecord.onModified )    
+        self.connect( self.form, PYSIGNAL("modified"), self.listViewRecord.onModified )
+        self.connect( self.image, PYSIGNAL("modified"), PYSIGNAL("modified") )
         # currently not needed: 
         # self.connect( self.image, PYSIGNAL("modified()"), self.onModified )
 
@@ -138,6 +136,21 @@ class RecordList(RecordListBase):
         self.listViewRecord.findNext(text, field, options)
 
 
+    def deletePicture(self):
+        """removes picture of current record"""
+        self.image.deletePicture()
+
+
+    def copyPicture(self):
+        """copy picture of current record"""
+        self.image.copyPicture()
+
+
+    def saveAsPicture(self):
+        """save picture on filesystem"""
+        self.image.saveAsPicture()
+    
+
     #-- slots ------------------------------------------------------------------
     def onGroup(self, group):
         """called when an online re-grouping is necessary"""
@@ -161,9 +174,11 @@ class RecordList(RecordListBase):
                 self.form.setRecord( record )
                 if (self.image.isHidden()): self.image.show()
                 self.image.setRecord( record )
+            self.emit(PYSIGNAL("currentChanged"), (record,))
         else:
-            self.form.setRecord(None)
-            self.image.setRecord(None)
+            self.form.hide()#setRecord(None)
+            self.image.hide()#setRecord(None)
+            self.emit(PYSIGNAL("currentChanged"), (None,))
 
 
     #-- utility functions ------------------------------------------------------
