@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Author:   $Author: merkosh $
-# Revision: $Rev: 128 $
+# Revision: $Rev: 153 $
 ############################################################################
 #    Copyright (C) 2005 by Uwe Mayer                                       #
 #    merkosh@hadiko.de                                                     #
@@ -35,7 +35,7 @@ from pprint import pformat
 CONFIG_TEXT = """\
 # -*- coding: utf-8 -*-
 # Author:   $Author: merkosh $
-# Revision: $Rev: 128 $
+# Revision: $Rev: 153 $
 ############################################################################
 #    Copyright (C) 2005 by Uwe Mayer                                       #
 #    merkosh@hadiko.de                                                     #
@@ -111,6 +111,15 @@ logLevel = %(logLevel)s
 defaultPath = %(defaultPath)s
 
 
+# reopenLastFile is a boolean value. If set to True the following
+# defaultFile option will be automatically updated to contain the
+# filenames opened. This causes the previous last file to be reopened
+# on startup.
+# Default: True
+
+reopenLastFile = %(reopenLastFile)s
+
+
 # defaultFile defines the file to be opened by lmc when it starts up.
 # The value can either be the path to a filename or None, in which case
 # the auto-open is disabled.
@@ -118,6 +127,13 @@ defaultPath = %(defaultPath)s
 defaultFile = %(defaultFile)s
 
 
+# recentFiles is a list of the four most recent files that have been
+# opened. These are maintained automatically, but the list can be
+# set to an empty list in order to remove all entries.
+
+recentFiles = %(recentFiles)s
+
+
 # defaultMoviePath defines the path which lmc uses as its first entry
 # point when getting information from files
 # The default is the current working directory.
@@ -387,7 +403,11 @@ class _Settings0(dict):
                      },
 
         'defaultPath': os.environ['PWD'],
+        'reopenLastFile': True,
         'defaultFile': None,
+
+        'recentFiles': [],
+        
         'defaultMoviePath': os.environ['PWD'],
         'browserPath': '/usr/bin/mozilla',
 
@@ -505,7 +525,8 @@ class _Settings0(dict):
                     'zeroPrefix_patch', 'scripts', 'defaultScript', 'autoMountPoint',
                     'ejectOnUnmount', 'defaultHistory', 'addHistoryFromRecords',
                     'recordDefault', 'autoScanAutoMountOnNew', 'autoScanOnNew',
-                    'autoScriptOnNew', 'deepcopyOnCopy', 'defaultFile']:
+                    'autoScriptOnNew', 'deepcopyOnCopy', 'defaultFile',
+                    'reopenLastFile', 'recentFiles']:
             defs[key] = pformat(self[key])
 
         # class instance 
@@ -515,7 +536,6 @@ class _Settings0(dict):
         return defs
 
 
-
 #-- main class for config format -----------------------------------------------
 # If the configuration format needs to be changed and in order to keep
 # a transparent layer of compatibility for seemless upgrade the following
@@ -620,7 +640,8 @@ class _Pref(dict):
                     'defaultScript', 'autoMountPoint', 'ejectOnUnmount',
                     'defaultHistory', 'addHistoryFromRecords', 'recordDefault',
                     'autoScanAutoMountOnNew', 'autoScanOnNew', 'autoScriptOnNew',
-                    'deepcopyOnCopy', 'defaultFile']:
+                    'deepcopyOnCopy', 'defaultFile', 'reopenLastFile',
+                    'recentFiles']:
             self[key] = deepcopy(pref[key])
 
         # debug info on a per-class basis
