@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 # Author:   $Author: merkosh $
-# Revision: $Rev: 132 $
+# Revision: $Rev: 160 $
 ############################################################################
 #    Copyright (C) 2005 by Uwe Mayer                                       #
 #    merkosh@hadiko.de                                                     #
@@ -49,7 +49,7 @@ log.addHandler(hdlr)
 log.setLevel(logging.DEBUG)             # default log level
 
 # output program text-logo
-log.info("lmc.py - LMC Movie Catalogue  $Rev: 132 $ (c) by Uwe Mayer")
+log.info("lmc.py - LMC Movie Catalogue  $Rev: 160 $ (c) by Uwe Mayer")
 
 
 #-- own imports --
@@ -111,15 +111,21 @@ fileFormats = [(AMCFile33, 'AMC v3.3 (*.amc)', '.amc'),
 fileFilterList = filter(lambda a: a != "", map(lambda a: a[1], fileFormats))        
 
 
+# defines the start index in the file menu where the list of recent
+# files is inserted
+RECENT_FILES_START_INDEX = 13
+MAX_COUNT_RECENT_FILES = 4
+
+
 #-- utility functions ----------------------------------------------------------
-defaultCaption = "Moviefly ($Rev: 132 $) "
+defaultCaption = "Moviefly ($Rev: 160 $)"
 filenameCaption = lambda a: defaultCaption+" - %s"%basename(a)
 
 
 #-- main class -----------------------------------------------------------------
 class LMCMain(LMCWidgetBase):
     """main program class"""
-    def __init__(self):
+    def __init__(self, input_file):
         LMCWidgetBase.__init__(self)
 
         # read in config file
@@ -128,10 +134,13 @@ class LMCMain(LMCWidgetBase):
         except IOError, e:
             pass                        # TODO: do abort
 
+        # create logging
+        self.log = logging.getLogger("LMCMain")
+
         # scan from file popup menu
         self.movieGetLocalPopup = QPopupMenu(self)
         self.movieGetLocalPopup.insertItem(QIconSet(self.image13), self.tr("from files..."), self.movieGetLocalFromFiles)
-        self.movieGetLocalPopup.insertItem(QIconSet(self.image31), self.tr("from files on CD..."), self.movieGetLocalCDFromFiles)
+        self.movieGetLocalPopup.insertItem(QIconSet(self.image30), self.tr("from files on CD..."), self.movieGetLocalCDFromFiles)
         # disable actions
         self.movieGetLocalPopup.setItemEnabled(self.movieGetLocalPopup.idAt(0), True)
         self.movieGetLocalPopup.setItemEnabled(self.movieGetLocalPopup.idAt(1), False)
@@ -167,16 +176,19 @@ class LMCMain(LMCWidgetBase):
         # add popup menu
         self.toolGroupAction.setPopup(self.groupPopup)
 
+        # populate recent-files-list
+        self.recentFilesIndex = RECENT_FILES_START_INDEX
+        self._updateRecentFiles()
+
         # runtime variables
         self.__file = None              # no file opened so far
         self.lastFolder = pref['defaultPath']
         self.lastMovieFolder = pref['defaultMoviePath']
         self.recordCopy = None
         self.modified = False           # modified metadata or deleted record
+        self.recordActionsEnabled = False
+        self.pictureActionsEnabled = False
         
-        # create logging
-        self.log = logging.getLogger("LMCMain")
-
         # assign slots
         self.connect(self.fileOpenAction, SIGNAL("activated()"), self.fileOpen)
         self.fileOpenAction.setEnabled(True)
@@ -189,12 +201,13 @@ class LMCMain(LMCWidgetBase):
         self.toolNewAction.setEnabled(True)
 
         # auto-open
-        if (pref['defaultFile'] != None):
-            self.log.info("Opening default file %s", basename(pref['defaultFile']))
-            if (not fexists(pref['defaultFile'])):
-                self.log.warning("Cannot open file %s; file does not exist", basename(pref['defaultFile']))
+        infile = input_file or pref['defaultFile']
+        if (infile != None):
+            self.log.info("Opening default file %s", basename(infile))
+            if (not fexists(infile)):
+                self.log.warning("Cannot open file %s; file does not exist", basename(infile))
             else:
-                self.fileOpen(pref['defaultFile'])
+                self.fileOpen(infile)
 
 
 
@@ -210,21 +223,83 @@ class LMCMain(LMCWidgetBase):
         self.statusBar().message(message, delay)
 
 
+    def currentChanged(self, obj):
+        """called when the current item is changed"""
+        # this is used to connect and disconnect special actions which
+        # are only available when a record is selected        
+        # disconnect record-signals
+        if (obj == None):
+            if (self.recordActionsEnabled): 
+                self._disableRecordActions()
+            if (self.pictureActionsEnabled):
+                self._disablePictureActions()
+            
+        # connect record-signals
+        else:
+            if (not self.recordActionsEnabled):
+                self._enableRecordActions()
+
+            # check wether to enable picture actions
+            rec = self.centralWidget().getCurrentRecord()
+            if ((rec['embeddedImage'] or rec['picture']) and not self.pictureActionsEnabled):
+                self._enablePictureActions()
+
+            elif (not (rec['embeddedImage'] or rec['picture']) and self.pictureActionsEnabled):
+                self._disablePictureActions()
+
+
+    def onModified(self):
+        """called when the record was modified; currently: only picture"""
+        # check wether to enable picture actions
+        rec = self.centralWidget().getCurrentRecord()
+        if ((rec['embeddedImage'] or rec['picture']) and not self.pictureActionsEnabled):
+            self._enablePictureActions()
+
+        elif (not (rec['embeddedImage'] or rec['picture']) and self.pictureActionsEnabled):
+            self._disablePictureActions()
+        
+
     def closeEvent(self, e):
         """called when application is closed: checks if file needs saving"""
         if (self.__file != None):
             self.fileClose()
+
+        else:
+            # keep track of lastFileOpened configuration setting
+            if (pref['reopenLastFile']):
+                settings['defaultFile'] = None
+                Settings.updatePreferences()
+
+        settings.save()
+        self.log.info("Quitting")
         e.accept()
         
 
+    def fileOpenRecent(self, id=None):
+        """called when a recently opened file should be opened"""
+        # update global settings object
+        filename = settings['recentFiles'].pop(id -RECENT_FILES_START_INDEX)
+        settings['recentFiles'].insert(0, filename)
+        Settings.updatePreferences()
+        # update file menu
+        self._updateRecentFiles()
+        # open file
+        self.fileOpen(filename)
+
+
     def fileClose(self):
         """closes a file and removes the central widget"""
         self.log.info("Closing file %s", self.__file.name)
+        
         # remove slots
-        self.__disableSaveCloseActions()
-        if (len(self.centralWidget()) != 0): self.__disableMovieDeleteAction()
+        self.disconnect(self.centralWidget(), PYSIGNAL("currentChanged"), self.currentChanged)
+        if (self.pictureActionsEnabled):
+            self._disablePictureActions()
+        if (self.recordActionsEnabled):
+            self._disableRecordActions()
+        self._disableFileActions()
 
-        self.__querySaveOnDirty()
+        self._querySaveOnDirty()
 
         self.infoStatusBar("Closing file %s"%basename(self.__file.name))
         self.centralWidget().hide()
@@ -275,7 +350,7 @@ class LMCMain(LMCWidgetBase):
         filename = unicode(dlg.selectedFile())
 
         # get format class and filename
-        (cls, filename) = self.__getFormatByFilename(filename, dlg.selectedFilter())
+        (cls, filename) = self._getFormatByFilename(filename, dlg.selectedFilter())
 
         # test if saving as already opened file
         if (fexists(filename) and samefile(filename, self.__file.name)):
@@ -322,7 +397,7 @@ class LMCMain(LMCWidgetBase):
 
     def fileNew(self):
         """handles the create-new-file process"""
-        self.__querySaveOnDirty()
+        self._querySaveOnDirty()
         
         # create file dialog and get filename
         dlg = QFileDialog(self.lastFolder, ";;".join(fileFilterList), self)
@@ -334,7 +409,7 @@ class LMCMain(LMCWidgetBase):
         filename = unicode(dlg.selectedFile())
 
         # get format class and filename
-        (cls, filename) = self.__getFormatByFilename(filename, dlg.selectedFilter())
+        (cls, filename) = self._getFormatByFilename(filename, dlg.selectedFilter())
 
         # test wether file already exists
         if (fexists(filename)):
@@ -362,20 +437,38 @@ class LMCMain(LMCWidgetBase):
         self.log.info("Creating new file %s", basename(filename))
         self.__file = f
 
+        # keep track of lastFileOpened configuration setting
+        if (pref['reopenLastFile']):
+            settings['defaultFile'] = filename
+            Settings.updatePreferences()
+
+        # keep track of recent files opened
+        # update global settings for recentFiles
+        settings['recentFiles'] = filter(lambda a: a != filename, pref['recentFiles'])
+        if (len(settings['recentFiles']) >= MAX_COUNT_RECENT_FILES):
+            settings['recentFiles'].pop()
+        settings['recentFiles'].insert(0, filename)
+        Settings.updatePreferences()
+        # update file menu
+        self._updateRecentFiles()
+
         # set caption
         self.setCaption(filenameCaption(self.__file.name))
 
         # instanciate and new, empty center widget
         self.setCentralWidget( RecordList(dirname(filename), self) )
         self.centralWidget().show()
+
         # connect slots
-        self.__enableSaveCloseActions()
+        self._enableFileActions()
+        self.connect(self.centralWidget(), PYSIGNAL("currentChanged"), self.currentChanged)
+        
 
 
     def fileOpen(self, defaultFile=None):
         """handles the open-existing-file process"""
         # query save when records dirty
-        self.__querySaveOnDirty()
+        self._querySaveOnDirty()
 
         # no default open given: load from dialog
         if (defaultFile == None):
@@ -429,18 +522,37 @@ class LMCMain(LMCWidgetBase):
         self.log.info("Opening file %s (size: %d)", basename(filename), fsize(filename))
         self.__file = f
 
+        # keep track of lastFileOpened configuration setting
+        if (pref['reopenLastFile']):
+            settings['defaultFile'] = filename
+            Settings.updatePreferences()
+
+        # keep track of recent files opened
+        # update global settings for recentFiles
+        settings['recentFiles'] = filter(lambda a: a != filename, pref['recentFiles'])
+        if (len(settings['recentFiles']) >= MAX_COUNT_RECENT_FILES):
+            settings['recentFiles'].pop()
+        settings['recentFiles'].insert(0, filename)
+        Settings.updatePreferences()
+        # update file menu
+        self._updateRecentFiles()
+
         # set caption
         self.setCaption(filenameCaption(self.__file.name))
 
         # instanciate and load new center widget
         self.setCentralWidget( RecordList(self.lastFolder, self) )
         self.centralWidget().add( self.__file.read() )
+
+        # connect currentChanged signal now so that focusFirst causes
+        # picture and record actions to be enabled
+        self.connect(self.centralWidget(), PYSIGNAL("currentChanged"), self.currentChanged)
+
         self.centralWidget().focusFirst()
         self.centralWidget().show()
-        # connect slots
-        self.__enableSaveCloseActions()
-        if (len(self.centralWidget()) > 0):
-            self.__enableMovieDeleteAction()
+
+        # enable rest of the actions
+        self._enableFileActions()
 
 
     def fileProperties(self):
@@ -478,9 +590,9 @@ class LMCMain(LMCWidgetBase):
         self.centralWidget().add(record)
         self.centralWidget().focusNew()
 
-        if (len(self.centralWidget()) == 1):
-            self.__enableMovieDeleteAction()
-
+        # info: connecting record-actions not necessary; they will be
+        # connected as part of the "current-changed" signal
+            
         # do autoScanOnNew
         if (pref['autoScanOnNew']):
             self.movieGetLocalFromFiles()
@@ -503,8 +615,21 @@ class LMCMain(LMCWidgetBase):
 
         self.modified = True
 
-        if (len(self.centralWidget()) == 0):
-            self.__disableMovieDeleteAction()
+
+    def moviePictureDelete(self):
+        """delete picture of current record"""
+        # delegate to central widget
+        self.centralWidget().deletePicture()
+
+
+    def moviePictureCopy(self):
+        """copies current picture"""
+        self.centralWidget().copyPicture()
+        
+
+    def moviePictureSaveAs(self):
+        """copies current picture"""
+        self.centralWidget().saveAsPicture()
         
 
     def movieGetLocalFromFiles(self):
@@ -533,7 +658,6 @@ class LMCMain(LMCWidgetBase):
         record['videoFormat'] = info.videoFormat
         record['audioFormat'] = info.audioFormat
         record['resolution'] = info.resolution
-#        record['languages'] = info.languages
         record['subtitles'] = info.subtitles
         record['videoBitrate'] = info.videoBitrate
         record['audioBitrate'] = info.audioBitrate
@@ -556,12 +680,11 @@ class LMCMain(LMCWidgetBase):
         self.centralWidget().form.focus("mediaLabel")
 
 
-
     def movieGetLocalCDFromFiles(self):
         """mount /unmount when getting information from a file"""
         # mount target volume
         self.log.info("Automounting %s", pref['autoMountPoint'])
-        os.system("mount %s"%pref['autoMountPoint'])
+        ret = os.system("mount %s"%pref['autoMountPoint'])
         
         # remember usual default target path
         tmp = self.lastMovieFolder
@@ -574,11 +697,17 @@ class LMCMain(LMCWidgetBase):
 
         # unmount or eject
         if (pref['ejectOnUnmount']):
-            self.log.info("Ejecting %s", pref['autoMountPoint'])
-            os.system("eject %s &"%pref['autoMountPoint'])
+            if (ret != 0):
+                self.log.info("mount exited with non-zero exit code; not ejecting")
+            else:
+                self.log.info("Ejecting %s", pref['autoMountPoint'])
+                os.system("eject %s &"%pref['autoMountPoint'])
         else:
-            self.log.info("Unmounting %s", pref['autoMountPoint'])
-            os.system("umount %s &"%pref['autoMountPoint'])
+            if (ret != 0):
+                self.log.info("mount exited with non-zero exit code; not unmounting")
+            else:
+                self.log.info("Unmounting %s", pref['autoMountPoint'])
+                os.system("umount %s &"%pref['autoMountPoint'])
        
 
     def movieGetLocalFromScript(self):
@@ -776,7 +905,29 @@ class LMCMain(LMCWidgetBase):
 
 
     #-- helper methods ---------------------------------------------------------
-    def __querySaveOnDirty(self):
+    def _updateRecentFiles(self):
+        # remove recent files in the menu
+        while (self.recentFilesIndex > RECENT_FILES_START_INDEX):
+            self.recentFilesIndex -= 1
+            self.fileMenu.removeItem(self.recentFilesIndex)
+
+        # remove recent files which don't exist
+        for i in range(len(settings['recentFiles'])-1, -1, -1):
+            if (not fexists(settings['recentFiles'][i])):
+                self.log.info("Removing recent file %s: file does not exist", basename(settings['recentFiles'][i]))
+                del settings['recentFiles'][i]
+
+        # add to menu
+        for f in settings['recentFiles']:
+            if (f[:len(os.environ['HOME'])] == os.environ['HOME']):
+                f = f.replace(os.environ['HOME'], '~')
+            if (len(f) > 15+3+15):
+                f = f[:15]+"..."+f[-15:]
+            self.fileMenu.insertItem(f, self.fileOpenRecent, 0, self.recentFilesIndex, self.recentFilesIndex)
+            self.recentFilesIndex += 1
+    
+
+    def _querySaveOnDirty(self):
         """asks user wether to save file when there are dirty records"""
         # check if currently opened records were modified
         if (self.__file != None):
@@ -799,7 +950,7 @@ class LMCMain(LMCWidgetBase):
 
 
     # used by SaveAs and New to decide on final file format and filename
-    def __getFormatByFilename(self, filename, selectedFilter):
+    def _getFormatByFilename(self, filename, selectedFilter):
         """returns file format class and final filename from filename and selectedFilter"""
         cls = None                      # takes up target class
         # 1. extension
@@ -831,8 +982,10 @@ class LMCMain(LMCWidgetBase):
         
         
 
-    def __enableSaveCloseActions(self):
+    def _enableFileActions(self):
         """enables Save, SaveAs and Close actions"""
+        self.log.debug("Enabling file actions")
+        
         self.connect(self.fileCloseAction, SIGNAL("activated()"), self.fileClose)
         self.fileCloseAction.setEnabled(True)
         
@@ -854,9 +1007,20 @@ class LMCMain(LMCWidgetBase):
         self.connect(self.filePropertiesAction, SIGNAL("activated()"), self.fileProperties)
         self.filePropertiesAction.setEnabled(True)
 
+        self.connect(self.movieFindAction, SIGNAL("activated()"), self.movieFind)
+        self.movieFindAction.setEnabled(True)
 
-    def __disableSaveCloseActions(self):
+        self.connect(self.toolFindAction, SIGNAL("pressed()"), self.movieFind)
+        self.toolFindAction.setEnabled(True)
+
+        self.connect(self.toolGroupAction, SIGNAL("clicked()"), self.movieGroup)
+        self.toolGroupAction.setEnabled(True)
+
+
+    def _disableFileActions(self):
         """disables Save, SaveAs and Close actions"""
+        self.log.debug("Disabling file actions")
+        
         self.fileCloseAction.setEnabled(False)
         self.disconnect(self.fileCloseAction, SIGNAL("activated()"), self.fileClose)
 
@@ -887,9 +1051,21 @@ class LMCMain(LMCWidgetBase):
         self.filePropertiesAction.setEnabled(False)
         self.disconnect(self.filePropertiesAction, SIGNAL("activated()"), self.fileProperties)
 
+        self.movieFindAction.setEnabled(False)
+        self.disconnect(self.movieFindAction, SIGNAL("activated()"), self.movieFind)
 
-    def __enableMovieDeleteAction(self):
+        self.toolFindAction.setEnabled(False)
+        self.disconnect(self.toolFindAction, SIGNAL("pressed()"), self.movieFind)
+
+        self.toolGroupAction.setEnabled(False)
+        self.disconnect(self.toolGroupAction, SIGNAL("clicked()"), self.movieGroup)
+
+
+    def _enableRecordActions(self):
         """enables actions related to deleting a record"""
+        self.log.debug("Enabling record actions")
+        assert(self.recordActionsEnabled == False)
+
         self.connect(self.movieDeleteAction, SIGNAL("activated()"), self.movieDelete)
         self.movieDeleteAction.setEnabled(True)
 
@@ -916,6 +1092,7 @@ class LMCMain(LMCWidgetBase):
         self.toolGetFromScriptAction.setEnabled(True)
 
         self.connect(self.centralWidget(), PYSIGNAL("searchTextChanged"), self.updateStatusBar)
+        self.connect(self.centralWidget(), PYSIGNAL("modified"), self.onModified)
 
         self.connect(self.movieCopyAction, SIGNAL("activated()"), self.movieCopy)
         self.movieCopyAction.setEnabled(True)
@@ -923,19 +1100,15 @@ class LMCMain(LMCWidgetBase):
         self.connect(self.toolCopyAction, SIGNAL("pressed()"), self.movieCopy)
         self.toolCopyAction.setEnabled(True)
 
-        self.connect(self.movieFindAction, SIGNAL("activated()"), self.movieFind)
-        self.movieFindAction.setEnabled(True)
-
-        self.connect(self.toolFindAction, SIGNAL("pressed()"), self.movieFind)
-        self.toolFindAction.setEnabled(True)
-
-        self.connect(self.toolGroupAction, SIGNAL("clicked()"), self.movieGroup)
-        self.toolGroupAction.setEnabled(True)
+        self.recordActionsEnabled = True
         
 
 
-    def __disableMovieDeleteAction(self):
+    def _disableRecordActions(self):
         """disables actions related to deleting a record"""
+        self.log.debug("Disabling record actions")
+        assert(self.recordActionsEnabled == True)
+
         self.movieDeleteAction.setEnabled(False)
         self.disconnect(self.movieDeleteAction, SIGNAL("activated()"), self.movieDelete)
 
@@ -961,6 +1134,7 @@ class LMCMain(LMCWidgetBase):
         self.disconnect(self.toolGetFromScriptAction, SIGNAL("pressed()"), self.movieGetLocalFromScript)
 
         self.disconnect(self.centralWidget(), PYSIGNAL("searchTextChanged"), self.updateStatusBar)
+        self.disconnect(self.centralWidget(), PYSIGNAL("modified"), self.onModified)
 
         self.movieCopyAction.setEnabled(False)
         self.disconnect(self.movieCopyAction, SIGNAL("activated()"), self.movieCopy)
@@ -968,23 +1142,68 @@ class LMCMain(LMCWidgetBase):
         self.toolCopyAction.setEnabled(False)
         self.disconnect(self.toolCopyAction, SIGNAL("pressed()"), self.movieCopy)
 
-        self.movieFindAction.setEnabled(False)
-        self.disconnect(self.movieFindAction, SIGNAL("activated()"), self.movieFind)
+        self.recordActionsEnabled = False
 
-        self.toolFindAction.setEnabled(False)
-        self.disconnect(self.toolFindAction, SIGNAL("pressed()"), self.movieFind)
 
-        self.toolGroupAction.setEnabled(False)
-        self.disconnect(self.toolGroupAction, SIGNAL("clicked()"), self.movieGroup)
-        
-        
+    def _enablePictureActions(self):
+        """enables actions on a picture"""
+        self.log.debug("Enabling picture actions")
+        assert(self.pictureActionsEnabled == False)
+
+        self.connect(self.moviePictureDeleteAction, SIGNAL("activated()"), self.moviePictureDelete)
+        self.moviePictureDeleteAction.setEnabled(True)
+
+        self.connect(self.moviePictureCopyAction, SIGNAL("activated()"), self.moviePictureCopy)
+        self.moviePictureCopyAction.setEnabled(True)
+
+        self.connect(self.moviePictureSaveAsAction, SIGNAL("activated()"), self.moviePictureSaveAs)
+        self.moviePictureSaveAsAction.setEnabled(True)
+
+        self.pictureActionsEnabled = True
+
+
+    def _disablePictureActions(self):
+        """disables actions on a picture"""
+        self.log.debug("Disabling picture actions")
+        assert(self.pictureActionsEnabled == True)
+
+        self.moviePictureDeleteAction.setEnabled(False)
+        self.disconnect(self.moviePictureDeleteAction, SIGNAL("activated()"), self.moviePictureDelete)
+
+        self.moviePictureCopyAction.setEnabled(False)
+        self.disconnect(self.moviePictureCopyAction, SIGNAL("activated()"), self.moviePictureCopy)
+
+        self.moviePictureSaveAsAction.setEnabled(False)
+        self.disconnect(self.moviePictureSaveAsAction, SIGNAL("activated()"), self.moviePictureSaveAs)
+
+        self.pictureActionsEnabled = False
+
 
 #-- main program ---------------------------------------------------------------
 app = QApplication(sys.argv)
 
+#-- parse command line arguments
+# help text
+if (('-h' in app.argv()[1:]) or ('--help' in app.argv()[1:])):
+    print """
+Synopsis: lmc.py [-h|--help] [<input-file>]
+
+   -h       --help       this screen
+   <input-file>          open this file on startup
+                         This option overwrites the default startup file from the
+                         program configuration.
+"""
+    sys.exit()
+
+# input file
+infile = None
+if (len(app.argv()) > 1):
+    infile = app.argv()[1]
+
+
 QObject.connect(app, SIGNAL("lastWindowClosed()"), app, SLOT("quit()"))
 
-main = LMCMain()
+main = LMCMain(infile)
 
 app.setMainWidget(main)
 main.show()
